@@ -26,6 +26,7 @@ import struct
 import sys
 import os
 from util.trace import Trace
+from util.translate import *
 from gen.container import *
 from io.path import *
 
@@ -34,6 +35,9 @@ class Image(Container):
   "An embedded image"
 
   ignoredtexts = ImageConfig.size['ignoredtexts']
+  vectorformats = ImageConfig.formats['vector']
+  rasterformats = ImageConfig.formats['raster']
+  defaultformat = ImageConfig.formats['default']
 
   def __init__(self):
     self.parser = InsetParser()
@@ -41,6 +45,9 @@ class Image(Container):
     self.type = 'embedded'
     self.width = None
     self.height = None
+    self.maxwidth = None
+    self.maxheight = None
+    self.scale = None
 
   def process(self):
     "Place the url, convert the image if necessary."
@@ -49,6 +56,7 @@ class Image(Container):
       Trace.error('Image ' + unicode(self.origin) + ' not found')
       return
     self.destination = self.getdestination(self.origin)
+    self.setscale()
     ImageConverter.instance.convert(self)
     self.setsize()
 
@@ -57,7 +65,7 @@ class Image(Container):
     "Changes extension of destination to output image format."
     destination = OutputPath(origin)
     forceformat = '.jpg'
-    forcedest = '.png'
+    forcedest = Image.defaultformat
     if Options.forceformat:
       forceformat = Options.forceformat
       forcedest = Options.forceformat
@@ -66,18 +74,24 @@ class Image(Container):
     destination.removebackdirs()
     return destination
 
+  def setscale(self):
+    "Set the scale attribute if present."
+    self.setifparam('scale')
+
   def setsize(self):
     "Set the size attributes width and height."
-    self.setifparam('width')
-    self.setifparam('height')
-    if self.width or self.height:
-      return
     imagefile = ImageFile(self.destination)
     width, height = imagefile.getdimensions()
     if width:
-      self.width = unicode(width)
+      self.maxwidth = unicode(width) + 'px'
+      if self.scale:
+        self.width = self.scalevalue(width)
     if height:
-      self.height = unicode(height)
+      self.maxheight = unicode(height) + 'px'
+      if self.scale:
+        self.height = self.scalevalue(height)
+    self.setifparam('width')
+    self.setifparam('height')
 
   def setifparam(self, name):
     "Set the value in the container if it exists as a param."
@@ -89,6 +103,11 @@ class Image(Container):
         value = value.replace(ignored, '')
     setattr(self, name, value)
 
+  def scalevalue(self, value):
+    "Scale the value according to the image scale and return it as unicode."
+    scaled = value * int(self.scale) / 100
+    return unicode(int(scaled)) + 'px'
+
 class ImageConverter(object):
   "A converter from one image file to another."
 
@@ -99,7 +118,7 @@ class ImageConverter(object):
     "Convert an image to PNG"
     if not ImageConverter.active:
       return
-    if image.origin == image.destination:
+    if image.origin.path == image.destination.path:
       return
     if image.destination.exists():
       if image.origin.getmtime() <= image.destination.getmtime():
@@ -128,13 +147,13 @@ class ImageConverter(object):
   def getparams(self, image):
     "Get the parameters for ImageMagick conversion"
     params = dict()
-    scale = 100
-    if 'scale' in image.parameters:
-      scale = int(image.parameters['scale'])
-    if image.origin.hasext('.svg'):
+    if image.origin.hasexts(Image.vectorformats):
+      scale = 100
+      if image.scale:
+        scale = image.scale
+        # descale
+        image.scale = None
       params['density'] = scale
-    elif image.origin.hasext('.jpg') or image.origin.hasext('.png'):
-      params['resize'] = unicode(scale) + '%'
     elif image.origin.hasext('.pdf'):
       params['define'] = 'pdf:use-cropbox=true'
     return params
@@ -225,7 +244,7 @@ class ImageFile(object):
 class ImageOutput(object):
   "Returns an image in the output"
 
-  figure = TranslationConfig.constants['figure']
+  figure = Translator.translate('figure')
 
   def gethtml(self, container):
     "Get the HTML output of the image as a list"
@@ -233,10 +252,16 @@ class ImageOutput(object):
     if container.origin.exists():
       html.append(' src="' + container.destination.url +
           '" alt="' + ImageOutput.figure + ' ' + container.destination.url + '"')
+      html.append(' style="')
       if container.width:
-        html.append(' width="' + container.width + '"')
+        html.append('width: ' + container.width + '; ')
+      if container.maxwidth:
+        html.append('max-width: ' + container.maxwidth + '; ')
       if container.height:
-        html.append(' height="' + container.height + '"')
+        html.append('height: ' + container.height + '; ')
+      if container.maxheight:
+        html.append('max-height: ' + container.maxheight + '; ')
+      html.append('"')
     else:
       html.append(' src="' + container.origin.url + '"')
     html.append('/>\n')
