@@ -52,10 +52,18 @@ class Layout(Container):
 class StandardLayout(Layout):
   "A standard layout -- can be a true div or nothing at all"
 
+  indentation = False
+
   def process(self):
     self.type = 'standard'
     self.output = ContentsOutput()
 
+  def complete(self, contents):
+    "Set the contents and return it."
+    self.process()
+    self.contents = contents
+    return self
+
 class Title(Layout):
   "The title of the whole document"
 
@@ -83,7 +91,7 @@ class Abstract(Layout):
   def process(self):
     self.type = 'abstract'
     self.output.tag = 'div class="abstract"'
-    message = TranslationConfig.constants['abstract']
+    message = Translator.translate('abstract')
     tagged = TaggedText().constant(message, 'p class="abstract-message"', True)
     self.contents.insert(0, tagged)
 
