@@ -35,6 +35,25 @@ class Table(Container):
   def __init__(self):
     self.parser = TableParser()
     self.output = TaggedOutput().settag('table', True)
+    self.columns = []
+
+  def process(self):
+    "Set the columns on every row"
+    index = 0
+    while index < len(self.contents):
+      element = self.contents[index]
+      if isinstance(element, Column):
+        self.columns.append(element)
+        del self.contents[index]
+      elif isinstance(element, BlackBox):
+        del self.contents[index]
+      elif isinstance(element, Row):
+        element.setcolumns(self.columns)
+        index += 1
+      else:
+        Trace.error('Unknown element type ' + element.__class__.__name__ +
+            ' in table: ' + unicode(element.contents[0]))
+        index += 1
 
 class Row(Container):
   "A row in a table"
@@ -47,15 +66,26 @@ class Row(Container):
   def setcolumns(self, columns):
     "Process alignments for every column"
     if len(columns) != len(self.contents):
-      Trace.error('Columns: ' + str(len(columns)) + ', cells: ' + str(len(self.contents)))
+      Trace.error('Columns: ' + unicode(len(columns)) + ', cells: ' + unicode(len(self.contents)))
       return
-    for index, column in enumerate(columns):
-      alignment = column['alignment']
-      if alignment == 'block':
-        alignment = 'justify'
-      self.contents[index].setalignment(alignment)
-      valignment = column['valignment']
-      self.contents[index].setvalignment(valignment)
+    for index, cell in enumerate(self.contents):
+      columns[index].set(cell)
+
+class Column(Container):
+  "A column definition in a table"
+
+  def __init__(self):
+    self.parser = ColumnParser()
+    self.output = EmptyOutput()
+
+  def set(self, cell):
+    "Set alignments in the corresponding cell"
+    alignment = self.parameters['alignment']
+    if alignment == 'block':
+      alignment = 'justify'
+    cell.setattribute('align', alignment)
+    valignment = self.parameters['valignment']
+    cell.setattribute('valign', valignment)
 
 class Cell(Container):
   "A cell in a table"
@@ -68,14 +98,6 @@ class Cell(Container):
     "Set the cell as multicolumn"
     self.setattribute('colspan', span)
 
-  def setalignment(self, alignment):
-    "Set the alignment for the cell"
-    self.setattribute('align', alignment)
-
-  def setvalignment(self, valignment):
-    "Set the vertical alignment"
-    self.setattribute('valign', valignment)
-
   def setattribute(self, attribute, value):
     "Set a cell attribute in the tag"
     self.output.tag += ' ' + attribute + '="' + unicode(value) + '"'
