@@ -116,7 +116,7 @@ class Bracket(FormulaBit):
   def parsecomplete(self, pos, innerparser):
     "Parse the start and end marks"
     if not pos.checkfor(self.start):
-      Trace.error('Bracket should start with ' + self.start + ' at ' + pos.remaining())
+      Trace.error('Bracket should start with ' + self.start + ' at ' + pos.identifier())
       return
     self.skiporiginal(self.start, pos)
     pos.pushending(self.ending)
@@ -133,7 +133,7 @@ class Bracket(FormulaBit):
     if pos.finished():
       return
     if pos.current() != self.ending:
-      Trace.error('No formula in bracket at ' + pos.remaining())
+      Trace.error('No formula in bracket at ' + pos.identifier())
     return
 
   def innertext(self, pos):
@@ -142,11 +142,9 @@ class Bracket(FormulaBit):
     while not pos.finished():
       if pos.current() == FormulaConfig.starts['command']:
         bit = factory.parsebit(pos)
-        if pos.current() == ' ':
-          pos.skip(pos.current())
+        pos.checkskip(' ')
       else:
-        bit = FormulaConstant(pos.current())
-        pos.skip(pos.current())
+        bit = FormulaConstant(pos.currentskip())
       self.add(bit)
 
   def innerliteral(self, pos):
