@@ -91,6 +91,12 @@ class Parser(object):
     while not reader.currentline().startswith(self.ending):
       process()
 
+  def parsecontainer(self, reader, contents):
+    container = self.factory.createcontainer(reader)
+    if container:
+      container.parent = self.parent
+      contents.append(container)
+
   def __unicode__(self):
     "Return a description"
     return self.__class__.__name__ + ' (' + unicode(self.begin) + ')'
@@ -119,8 +125,7 @@ class TextParser(Parser):
         ContainerConfig.endings['Inset'], self.ending]
     contents = []
     while not self.isending(reader):
-      containers = self.factory.createsome(reader)
-      contents += containers
+      self.parsecontainer(reader, contents)
     return contents
 
   def isending(self, reader):
@@ -142,13 +147,9 @@ class ExcludingParser(Parser):
   def parse(self, reader):
     "Parse everything up to (and excluding) the final line"
     contents = []
-    self.parseending(reader, lambda: self.parsecontainers(reader, contents))
+    self.parseending(reader, lambda: self.parsecontainer(reader, contents))
     return contents
 
-  def parsecontainers(self, reader, contents):
-    containers = self.factory.createsome(reader)
-    contents += containers
-
 class BoundedParser(ExcludingParser):
   "A parser bound by a final line"
 
@@ -206,7 +207,7 @@ class HeaderParser(Parser):
   def parseline(self, reader):
     "Parse a single line as a parameter or as a start"
     line = reader.currentline()
-    if line.startswith(ContainerConfig.header['branch']):
+    if line.startswith(HeaderConfig.parameters['branch']):
       self.parsebranch(reader)
       return
     # no match
@@ -215,7 +216,7 @@ class HeaderParser(Parser):
   def parsebranch(self, reader):
     branch = reader.currentline().split()[1]
     reader.nextline()
-    subparser = HeaderParser().complete(ContainerConfig.header['endbranch'])
+    subparser = HeaderParser().complete(HeaderConfig.parameters['endbranch'])
     subparser.parse(reader)
     options = BranchOptions(branch)
     for key in subparser.parameters:
