@@ -24,6 +24,7 @@
 
 import sys
 from gen.container import *
+from io.fileline import *
 from util.trace import Trace
 from conf.config import *
 
@@ -31,18 +32,32 @@ from conf.config import *
 class Position(object):
   "A position in a text to parse"
 
-  def __init__(self, text):
-    self.text = text
-    self.pos = 0
+  def __init__(self):
     self.endinglist = EndingList()
 
   def skip(self, string):
     "Skip a string"
-    self.pos += len(string)
+    Trace.error('Unimplemented skip()')
 
-  def remaining(self):
-    "Return the text remaining for parsing"
-    return self.text[self.pos:]
+  def identifier(self):
+    "Return an identifier for the current position."
+    Trace.error('Unimplemented identifier()')
+    return 'Error'
+
+  def isout(self):
+    "Find out if we are out of the position yet."
+    Trace.error('Unimplemented isout()')
+    return True
+
+  def current(self):
+    "Return the current character"
+    Trace.error('Unimplemented current()')
+    return ''
+
+  def checkfor(self, string):
+    "Check for a string at the given position."
+    Trace.error('Unimplemented checkfor()')
+    return False
 
   def finished(self):
     "Find out if the current formula has finished"
@@ -51,22 +66,11 @@ class Position(object):
       return True
     return self.endinglist.checkin(self)
 
-  def isout(self):
-    "Find out if we are out of the formula yet"
-    return self.pos >= len(self.text)
-
-  def current(self):
-    "Return the current character"
-    if self.isout():
-      Trace.error('Out of the formula')
-      return ''
-    return self.text[self.pos]
-
-  def checkfor(self, string):
-    "Check for a string at the given position"
-    if self.pos + len(string) > len(self.text):
-      return False
-    return self.text[self.pos : self.pos + len(string)] == string
+  def currentskip(self):
+    "Return the current character and skip it."
+    current = self.current()
+    self.skip(current)
+    return current
 
   def checkskip(self, string):
     "Check for a string at the given position; if there, skip it"
@@ -91,6 +95,16 @@ class Position(object):
     "Skip all whitespace at current position"
     return self.glob(lambda current: current.isspace())
 
+  def globincluding(self, magicchar):
+    "Glob a bit of text up to (including) the magic char."
+    glob = self.glob(lambda current: current != magicchar) + magicchar
+    self.skip(magicchar)
+    return glob
+
+  def globexcluding(self, magicchar):
+    "Glob a bit of text up until (excluding) the magic char."
+    return self.glob(lambda current: current != magicchar)
+
   def pushending(self, ending, optional = False):
     "Push a new ending to the bottom"
     self.endinglist.add(ending, optional)
@@ -103,6 +117,93 @@ class Position(object):
     self.skip(ending)
     return ending
 
+class TextPosition(Position):
+  "A parse position based on a raw text."
+
+  def __init__(self, text):
+    "Create the position from some text."
+    Position.__init__(self)
+    self.pos = 0
+    self.text = text
+
+  def skip(self, string):
+    "Skip a string of characters."
+    self.pos += len(string)
+
+  def identifier(self):
+    "Return a sample of the remaining text."
+    length = 30
+    if self.pos + length > len(self.text):
+      length = len(self.text) - self.pos - 1
+    return '*' + self.text[self.pos:self.pos + length]
+
+  def isout(self):
+    "Find out if we are out of the text yet."
+    return self.pos >= len(self.text)
+
+  def current(self):
+    "Return the current character, assuming we are not out."
+    return self.text[self.pos]
+
+  def checkfor(self, string):
+    "Check for a string at the given position."
+    if self.pos + len(string) > len(self.text):
+      return False
+    return self.text[self.pos : self.pos + len(string)] == string
+
+class FilePosition(Position):
+  "A parse position based on an underlying file."
+
+  def __init__(self, filename):
+    "Create the position from a file."
+    Position.__init__(self)
+    self.reader = LineReader(filename)
+    self.number = 1
+    self.pos = 0
+
+  def skip(self, string):
+    "Skip a string of characters."
+    length = len(string)
+    while self.pos + length > len(self.reader.currentline()):
+      length -= len(self.reader.currentline()) - self.pos + 1
+      self.nextline()
+    self.pos += length
+
+  def nextline(self):
+    "Go to the next line."
+    self.reader.nextline()
+    self.number += 1
+    self.pos = 0
+
+  def identifier(self):
+    "Return the current line and line number in the file."
+    before = self.reader.currentline()[:self.pos - 1]
+    after = self.reader.currentline()[self.pos:]
+    return 'line ' + unicode(self.number) + ': ' + before + '*' + after
+
+  def isout(self):
+    "Find out if we are out of the text yet."
+    if self.pos > len(self.reader.currentline()):
+      if self.pos > len(self.reader.currentline()) + 1:
+        Trace.error('Out of the line ' + self.reader.currentline() + ': ' + unicode(self.pos))
+      self.nextline()
+    return self.reader.finished()
+
+  def current(self):
+    "Return the current character, assuming we are not out."
+    if self.pos == len(self.reader.currentline()):
+      return '\n'
+    if self.pos > len(self.reader.currentline()):
+      Trace.error('Out of the line ' + self.reader.currentline() + ': ' + unicode(self.pos))
+      return '*'
+    return self.reader.currentline()[self.pos]
+
+  def checkfor(self, string):
+    "Check for a string at the given position."
+    if self.pos + len(string) > len(self.reader.currentline()):
+      return False
+    return self.reader.currentline()[self.pos : self.pos + len(string)] == string
+
 class EndingList(object):
   "A list of position endings"
 
