@@ -104,13 +104,14 @@ class PostBiblio(object):
 
   processedclass = Bibliography
 
-  def postprocess(self, element, last):
+  def postprocess(self, last, element, next):
     "If we have the first bibliography insert a tag"
     if isinstance(last, Bibliography):
       return element
-    bibliography = TranslationConfig.constants['bibliography']
-    tag = TaggedText().constant(bibliography, 'h1 class="biblio"')
-    return Group().contents([tag, element])
+    bibliography = Translator.translate('bibliography')
+    header = TaggedText().constant(bibliography, 'h1 class="biblio"')
+    layout = StandardLayout().complete([header, element])
+    return layout
 
 Postprocessor.stages.append(PostBiblio)
 
